@@ -1,4 +1,9 @@
 """Constants for AccuWeather integration."""
+from __future__ import annotations
+
+from typing import Final
+
+from homeassistant.components.sensor import ATTR_STATE_CLASS, STATE_CLASS_MEASUREMENT
 from homeassistant.components.weather import (
     ATTR_CONDITION_CLEAR_NIGHT,
     ATTR_CONDITION_CLOUDY,
@@ -17,6 +22,7 @@ from homeassistant.components.weather import (
 )
 from homeassistant.const import (
     ATTR_DEVICE_CLASS,
+    ATTR_ICON,
     CONCENTRATION_PARTS_PER_CUBIC_METER,
     DEVICE_CLASS_TEMPERATURE,
     LENGTH_FEET,
@@ -32,19 +38,23 @@ from homeassistant.const import (
     UV_INDEX,
 )
 
-ATTRIBUTION = "Data provided by AccuWeather"
-ATTR_ICON = "icon"
-ATTR_FORECAST = CONF_FORECAST = "forecast"
-ATTR_LABEL = "label"
-ATTR_UNIT_IMPERIAL = "Imperial"
-ATTR_UNIT_METRIC = "Metric"
-COORDINATOR = "coordinator"
-DOMAIN = "accuweather"
-MANUFACTURER = "AccuWeather, Inc."
-NAME = "AccuWeather"
-UNDO_UPDATE_LISTENER = "undo_update_listener"
+from .model import SensorDescription
 
-CONDITION_CLASSES = {
+API_IMPERIAL: Final = "Imperial"
+API_METRIC: Final = "Metric"
+ATTRIBUTION: Final = "Data provided by AccuWeather"
+ATTR_ENABLED: Final = "enabled"
+ATTR_FORECAST: Final = "forecast"
+ATTR_LABEL: Final = "label"
+ATTR_UNIT_IMPERIAL: Final = "unit_imperial"
+ATTR_UNIT_METRIC: Final = "unit_metric"
+CONF_FORECAST: Final = "forecast"
+DOMAIN: Final = "accuweather"
+MANUFACTURER: Final = "AccuWeather, Inc."
+MAX_FORECAST_DAYS: Final = 4
+NAME: Final = "AccuWeather"
+
+CONDITION_CLASSES: Final[dict[str, list[int]]] = {
     ATTR_CONDITION_CLEAR_NIGHT: [33, 34, 37],
     ATTR_CONDITION_CLOUDY: [7, 8, 38],
     ATTR_CONDITION_EXCEPTIONAL: [24, 30, 31],
@@ -52,24 +62,23 @@ CONDITION_CLASSES = {
     ATTR_CONDITION_HAIL: [25],
     ATTR_CONDITION_LIGHTNING: [15],
     ATTR_CONDITION_LIGHTNING_RAINY: [16, 17, 41, 42],
-    ATTR_CONDITION_PARTLYCLOUDY: [4, 6, 35, 36],
+    ATTR_CONDITION_PARTLYCLOUDY: [3, 4, 6, 35, 36],
     ATTR_CONDITION_POURING: [18],
     ATTR_CONDITION_RAINY: [12, 13, 14, 26, 39, 40],
     ATTR_CONDITION_SNOWY: [19, 20, 21, 22, 23, 43, 44],
     ATTR_CONDITION_SNOWY_RAINY: [29],
-    ATTR_CONDITION_SUNNY: [1, 2, 3, 5],
+    ATTR_CONDITION_SUNNY: [1, 2, 5],
     ATTR_CONDITION_WINDY: [32],
 }
 
-FORECAST_DAYS = [0, 1, 2, 3, 4]
-
-FORECAST_SENSOR_TYPES = {
+FORECAST_SENSOR_TYPES: Final[dict[str, SensorDescription]] = {
     "CloudCoverDay": {
         ATTR_DEVICE_CLASS: None,
         ATTR_ICON: "mdi:weather-cloudy",
         ATTR_LABEL: "Cloud Cover Day",
         ATTR_UNIT_METRIC: PERCENTAGE,
         ATTR_UNIT_IMPERIAL: PERCENTAGE,
+        ATTR_ENABLED: False,
     },
     "CloudCoverNight": {
         ATTR_DEVICE_CLASS: None,
@@ -77,6 +86,7 @@ FORECAST_SENSOR_TYPES = {
         ATTR_LABEL: "Cloud Cover Night",
         ATTR_UNIT_METRIC: PERCENTAGE,
         ATTR_UNIT_IMPERIAL: PERCENTAGE,
+        ATTR_ENABLED: False,
     },
     "Grass": {
         ATTR_DEVICE_CLASS: None,
@@ -84,6 +94,7 @@ FORECAST_SENSOR_TYPES = {
         ATTR_LABEL: "Grass Pollen",
         ATTR_UNIT_METRIC: CONCENTRATION_PARTS_PER_CUBIC_METER,
         ATTR_UNIT_IMPERIAL: CONCENTRATION_PARTS_PER_CUBIC_METER,
+        ATTR_ENABLED: False,
     },
     "HoursOfSun": {
         ATTR_DEVICE_CLASS: None,
@@ -91,6 +102,7 @@ FORECAST_SENSOR_TYPES = {
         ATTR_LABEL: "Hours Of Sun",
         ATTR_UNIT_METRIC: TIME_HOURS,
         ATTR_UNIT_IMPERIAL: TIME_HOURS,
+        ATTR_ENABLED: True,
     },
     "Mold": {
         ATTR_DEVICE_CLASS: None,
@@ -98,6 +110,7 @@ FORECAST_SENSOR_TYPES = {
         ATTR_LABEL: "Mold Pollen",
         ATTR_UNIT_METRIC: CONCENTRATION_PARTS_PER_CUBIC_METER,
         ATTR_UNIT_IMPERIAL: CONCENTRATION_PARTS_PER_CUBIC_METER,
+        ATTR_ENABLED: False,
     },
     "Ozone": {
         ATTR_DEVICE_CLASS: None,
@@ -105,6 +118,7 @@ FORECAST_SENSOR_TYPES = {
         ATTR_LABEL: "Ozone",
         ATTR_UNIT_METRIC: None,
         ATTR_UNIT_IMPERIAL: None,
+        ATTR_ENABLED: False,
     },
     "Ragweed": {
         ATTR_DEVICE_CLASS: None,
@@ -112,6 +126,7 @@ FORECAST_SENSOR_TYPES = {
         ATTR_LABEL: "Ragweed Pollen",
         ATTR_UNIT_METRIC: CONCENTRATION_PARTS_PER_CUBIC_METER,
         ATTR_UNIT_IMPERIAL: CONCENTRATION_PARTS_PER_CUBIC_METER,
+        ATTR_ENABLED: False,
     },
     "RealFeelTemperatureMax": {
         ATTR_DEVICE_CLASS: DEVICE_CLASS_TEMPERATURE,
@@ -119,6 +134,7 @@ FORECAST_SENSOR_TYPES = {
         ATTR_LABEL: "RealFeel Temperature Max",
         ATTR_UNIT_METRIC: TEMP_CELSIUS,
         ATTR_UNIT_IMPERIAL: TEMP_FAHRENHEIT,
+        ATTR_ENABLED: True,
     },
     "RealFeelTemperatureMin": {
         ATTR_DEVICE_CLASS: DEVICE_CLASS_TEMPERATURE,
@@ -126,6 +142,7 @@ FORECAST_SENSOR_TYPES = {
         ATTR_LABEL: "RealFeel Temperature Min",
         ATTR_UNIT_METRIC: TEMP_CELSIUS,
         ATTR_UNIT_IMPERIAL: TEMP_FAHRENHEIT,
+        ATTR_ENABLED: True,
     },
     "RealFeelTemperatureShadeMax": {
         ATTR_DEVICE_CLASS: DEVICE_CLASS_TEMPERATURE,
@@ -133,6 +150,7 @@ FORECAST_SENSOR_TYPES = {
         ATTR_LABEL: "RealFeel Temperature Shade Max",
         ATTR_UNIT_METRIC: TEMP_CELSIUS,
         ATTR_UNIT_IMPERIAL: TEMP_FAHRENHEIT,
+        ATTR_ENABLED: False,
     },
     "RealFeelTemperatureShadeMin": {
         ATTR_DEVICE_CLASS: DEVICE_CLASS_TEMPERATURE,
@@ -140,6 +158,7 @@ FORECAST_SENSOR_TYPES = {
         ATTR_LABEL: "RealFeel Temperature Shade Min",
         ATTR_UNIT_METRIC: TEMP_CELSIUS,
         ATTR_UNIT_IMPERIAL: TEMP_FAHRENHEIT,
+        ATTR_ENABLED: False,
     },
     "ThunderstormProbabilityDay": {
         ATTR_DEVICE_CLASS: None,
@@ -147,6 +166,7 @@ FORECAST_SENSOR_TYPES = {
         ATTR_LABEL: "Thunderstorm Probability Day",
         ATTR_UNIT_METRIC: PERCENTAGE,
         ATTR_UNIT_IMPERIAL: PERCENTAGE,
+        ATTR_ENABLED: True,
     },
     "ThunderstormProbabilityNight": {
         ATTR_DEVICE_CLASS: None,
@@ -154,6 +174,7 @@ FORECAST_SENSOR_TYPES = {
         ATTR_LABEL: "Thunderstorm Probability Night",
         ATTR_UNIT_METRIC: PERCENTAGE,
         ATTR_UNIT_IMPERIAL: PERCENTAGE,
+        ATTR_ENABLED: True,
     },
     "Tree": {
         ATTR_DEVICE_CLASS: None,
@@ -161,6 +182,7 @@ FORECAST_SENSOR_TYPES = {
         ATTR_LABEL: "Tree Pollen",
         ATTR_UNIT_METRIC: CONCENTRATION_PARTS_PER_CUBIC_METER,
         ATTR_UNIT_IMPERIAL: CONCENTRATION_PARTS_PER_CUBIC_METER,
+        ATTR_ENABLED: False,
     },
     "UVIndex": {
         ATTR_DEVICE_CLASS: None,
@@ -168,6 +190,7 @@ FORECAST_SENSOR_TYPES = {
         ATTR_LABEL: "UV Index",
         ATTR_UNIT_METRIC: UV_INDEX,
         ATTR_UNIT_IMPERIAL: UV_INDEX,
+        ATTR_ENABLED: True,
     },
     "WindGustDay": {
         ATTR_DEVICE_CLASS: None,
@@ -175,6 +198,7 @@ FORECAST_SENSOR_TYPES = {
         ATTR_LABEL: "Wind Gust Day",
         ATTR_UNIT_METRIC: SPEED_KILOMETERS_PER_HOUR,
         ATTR_UNIT_IMPERIAL: SPEED_MILES_PER_HOUR,
+        ATTR_ENABLED: False,
     },
     "WindGustNight": {
         ATTR_DEVICE_CLASS: None,
@@ -182,6 +206,7 @@ FORECAST_SENSOR_TYPES = {
         ATTR_LABEL: "Wind Gust Night",
         ATTR_UNIT_METRIC: SPEED_KILOMETERS_PER_HOUR,
         ATTR_UNIT_IMPERIAL: SPEED_MILES_PER_HOUR,
+        ATTR_ENABLED: False,
     },
     "WindDay": {
         ATTR_DEVICE_CLASS: None,
@@ -189,6 +214,7 @@ FORECAST_SENSOR_TYPES = {
         ATTR_LABEL: "Wind Day",
         ATTR_UNIT_METRIC: SPEED_KILOMETERS_PER_HOUR,
         ATTR_UNIT_IMPERIAL: SPEED_MILES_PER_HOUR,
+        ATTR_ENABLED: True,
     },
     "WindNight": {
         ATTR_DEVICE_CLASS: None,
@@ -196,37 +222,19 @@ FORECAST_SENSOR_TYPES = {
         ATTR_LABEL: "Wind Night",
         ATTR_UNIT_METRIC: SPEED_KILOMETERS_PER_HOUR,
         ATTR_UNIT_IMPERIAL: SPEED_MILES_PER_HOUR,
+        ATTR_ENABLED: True,
     },
 }
 
-OPTIONAL_SENSORS = (
-    "ApparentTemperature",
-    "CloudCover",
-    "CloudCoverDay",
-    "CloudCoverNight",
-    "DewPoint",
-    "Grass",
-    "Mold",
-    "Ozone",
-    "Ragweed",
-    "RealFeelTemperatureShade",
-    "RealFeelTemperatureShadeMax",
-    "RealFeelTemperatureShadeMin",
-    "Tree",
-    "WetBulbTemperature",
-    "WindChillTemperature",
-    "WindGust",
-    "WindGustDay",
-    "WindGustNight",
-)
-
-SENSOR_TYPES = {
+SENSOR_TYPES: Final[dict[str, SensorDescription]] = {
     "ApparentTemperature": {
         ATTR_DEVICE_CLASS: DEVICE_CLASS_TEMPERATURE,
         ATTR_ICON: None,
         ATTR_LABEL: "Apparent Temperature",
         ATTR_UNIT_METRIC: TEMP_CELSIUS,
         ATTR_UNIT_IMPERIAL: TEMP_FAHRENHEIT,
+        ATTR_ENABLED: False,
+        ATTR_STATE_CLASS: STATE_CLASS_MEASUREMENT,
     },
     "Ceiling": {
         ATTR_DEVICE_CLASS: None,
@@ -234,6 +242,8 @@ SENSOR_TYPES = {
         ATTR_LABEL: "Cloud Ceiling",
         ATTR_UNIT_METRIC: LENGTH_METERS,
         ATTR_UNIT_IMPERIAL: LENGTH_FEET,
+        ATTR_ENABLED: True,
+        ATTR_STATE_CLASS: STATE_CLASS_MEASUREMENT,
     },
     "CloudCover": {
         ATTR_DEVICE_CLASS: None,
@@ -241,6 +251,8 @@ SENSOR_TYPES = {
         ATTR_LABEL: "Cloud Cover",
         ATTR_UNIT_METRIC: PERCENTAGE,
         ATTR_UNIT_IMPERIAL: PERCENTAGE,
+        ATTR_ENABLED: False,
+        ATTR_STATE_CLASS: STATE_CLASS_MEASUREMENT,
     },
     "DewPoint": {
         ATTR_DEVICE_CLASS: DEVICE_CLASS_TEMPERATURE,
@@ -248,6 +260,8 @@ SENSOR_TYPES = {
         ATTR_LABEL: "Dew Point",
         ATTR_UNIT_METRIC: TEMP_CELSIUS,
         ATTR_UNIT_IMPERIAL: TEMP_FAHRENHEIT,
+        ATTR_ENABLED: False,
+        ATTR_STATE_CLASS: STATE_CLASS_MEASUREMENT,
     },
     "RealFeelTemperature": {
         ATTR_DEVICE_CLASS: DEVICE_CLASS_TEMPERATURE,
@@ -255,6 +269,8 @@ SENSOR_TYPES = {
         ATTR_LABEL: "RealFeel Temperature",
         ATTR_UNIT_METRIC: TEMP_CELSIUS,
         ATTR_UNIT_IMPERIAL: TEMP_FAHRENHEIT,
+        ATTR_ENABLED: True,
+        ATTR_STATE_CLASS: STATE_CLASS_MEASUREMENT,
     },
     "RealFeelTemperatureShade": {
         ATTR_DEVICE_CLASS: DEVICE_CLASS_TEMPERATURE,
@@ -262,6 +278,8 @@ SENSOR_TYPES = {
         ATTR_LABEL: "RealFeel Temperature Shade",
         ATTR_UNIT_METRIC: TEMP_CELSIUS,
         ATTR_UNIT_IMPERIAL: TEMP_FAHRENHEIT,
+        ATTR_ENABLED: False,
+        ATTR_STATE_CLASS: STATE_CLASS_MEASUREMENT,
     },
     "Precipitation": {
         ATTR_DEVICE_CLASS: None,
@@ -269,6 +287,8 @@ SENSOR_TYPES = {
         ATTR_LABEL: "Precipitation",
         ATTR_UNIT_METRIC: LENGTH_MILLIMETERS,
         ATTR_UNIT_IMPERIAL: LENGTH_INCHES,
+        ATTR_ENABLED: True,
+        ATTR_STATE_CLASS: STATE_CLASS_MEASUREMENT,
     },
     "PressureTendency": {
         ATTR_DEVICE_CLASS: "accuweather__pressure_tendency",
@@ -276,6 +296,7 @@ SENSOR_TYPES = {
         ATTR_LABEL: "Pressure Tendency",
         ATTR_UNIT_METRIC: None,
         ATTR_UNIT_IMPERIAL: None,
+        ATTR_ENABLED: True,
     },
     "UVIndex": {
         ATTR_DEVICE_CLASS: None,
@@ -283,6 +304,8 @@ SENSOR_TYPES = {
         ATTR_LABEL: "UV Index",
         ATTR_UNIT_METRIC: UV_INDEX,
         ATTR_UNIT_IMPERIAL: UV_INDEX,
+        ATTR_ENABLED: True,
+        ATTR_STATE_CLASS: STATE_CLASS_MEASUREMENT,
     },
     "WetBulbTemperature": {
         ATTR_DEVICE_CLASS: DEVICE_CLASS_TEMPERATURE,
@@ -290,6 +313,8 @@ SENSOR_TYPES = {
         ATTR_LABEL: "Wet Bulb Temperature",
         ATTR_UNIT_METRIC: TEMP_CELSIUS,
         ATTR_UNIT_IMPERIAL: TEMP_FAHRENHEIT,
+        ATTR_ENABLED: False,
+        ATTR_STATE_CLASS: STATE_CLASS_MEASUREMENT,
     },
     "WindChillTemperature": {
         ATTR_DEVICE_CLASS: DEVICE_CLASS_TEMPERATURE,
@@ -297,6 +322,8 @@ SENSOR_TYPES = {
         ATTR_LABEL: "Wind Chill Temperature",
         ATTR_UNIT_METRIC: TEMP_CELSIUS,
         ATTR_UNIT_IMPERIAL: TEMP_FAHRENHEIT,
+        ATTR_ENABLED: False,
+        ATTR_STATE_CLASS: STATE_CLASS_MEASUREMENT,
     },
     "Wind": {
         ATTR_DEVICE_CLASS: None,
@@ -304,6 +331,8 @@ SENSOR_TYPES = {
         ATTR_LABEL: "Wind",
         ATTR_UNIT_METRIC: SPEED_KILOMETERS_PER_HOUR,
         ATTR_UNIT_IMPERIAL: SPEED_MILES_PER_HOUR,
+        ATTR_ENABLED: True,
+        ATTR_STATE_CLASS: STATE_CLASS_MEASUREMENT,
     },
     "WindGust": {
         ATTR_DEVICE_CLASS: None,
@@ -311,5 +340,7 @@ SENSOR_TYPES = {
         ATTR_LABEL: "Wind Gust",
         ATTR_UNIT_METRIC: SPEED_KILOMETERS_PER_HOUR,
         ATTR_UNIT_IMPERIAL: SPEED_MILES_PER_HOUR,
+        ATTR_ENABLED: False,
+        ATTR_STATE_CLASS: STATE_CLASS_MEASUREMENT,
     },
 }
