@@ -59,14 +59,29 @@ async def async_setup_services(hass):
         service = service_call.service
         service_data = service_call.data
 
+        gateway = get_master_gateway(hass)
+        if CONF_BRIDGE_ID in service_data:
+            found_gateway = False
+            bridge_id = normalize_bridge_id(service_data[CONF_BRIDGE_ID])
+
+            for possible_gateway in hass.data[DOMAIN].values():
+                if possible_gateway.bridgeid == bridge_id:
+                    gateway = possible_gateway
+                    found_gateway = True
+                    break
+
+            if not found_gateway:
+                LOGGER.error("Could not find the gateway %s", bridge_id)
+                return
+
         if service == SERVICE_CONFIGURE_DEVICE:
-            await async_configure_service(hass, service_data)
+            await async_configure_service(gateway, service_data)
 
         elif service == SERVICE_DEVICE_REFRESH:
-            await async_refresh_devices_service(hass, service_data)
+            await async_refresh_devices_service(gateway)
 
         elif service == SERVICE_REMOVE_ORPHANED_ENTRIES:
-            await async_remove_orphaned_entries_service(hass, service_data)
+            await async_remove_orphaned_entries_service(gateway)
 
     hass.services.async_register(
         DOMAIN,
@@ -102,7 +117,7 @@ async def async_unload_services(hass):
     hass.services.async_remove(DOMAIN, SERVICE_REMOVE_ORPHANED_ENTRIES)
 
 
-async def async_configure_service(hass, data):
+async def async_configure_service(gateway, data):
     """Set attribute of device in deCONZ.
 
     Entity is used to resolve to a device path (e.g. '/lights/1').
@@ -118,10 +133,6 @@ async def async_configure_service(hass, data):
     See Dresden Elektroniks REST API documentation for details:
     http://dresden-elektronik.github.io/deconz-rest-doc/rest/
     """
-    gateway = get_master_gateway(hass)
-    if CONF_BRIDGE_ID in data:
-        gateway = hass.data[DOMAIN][normalize_bridge_id(data[CONF_BRIDGE_ID])]
-
     field = data.get(SERVICE_FIELD, "")
     entity_id = data.get(SERVICE_ENTITY)
     data = data[SERVICE_DATA]
@@ -136,31 +147,21 @@ async def async_configure_service(hass, data):
     await gateway.api.request("put", field, json=data)
 
 
-async def async_refresh_devices_service(hass, data):
+async def async_refresh_devices_service(gateway):
     """Refresh available devices from deCONZ."""
-    gateway = get_master_gateway(hass)
-    if CONF_BRIDGE_ID in data:
-        gateway = hass.data[DOMAIN][normalize_bridge_id(data[CONF_BRIDGE_ID])]
-
     gateway.ignore_state_updates = True
     await gateway.api.refresh_state()
     gateway.ignore_state_updates = False
 
-    gateway.async_add_device_callback(NEW_GROUP, force=True)
-    gateway.async_add_device_callback(NEW_LIGHT, force=True)
-    gateway.async_add_device_callback(NEW_SCENE, force=True)
-    gateway.async_add_device_callback(NEW_SENSOR, force=True)
+    for new_device_type in [NEW_GROUP, NEW_LIGHT, NEW_SCENE, NEW_SENSOR]:
+        gateway.async_add_device_callback(new_device_type, force=True)
 
 
-async def async_remove_orphaned_entries_service(hass, data):
+async def async_remove_orphaned_entries_service(gateway):
     """Remove orphaned deCONZ entries from device and entity registries."""
-    gateway = get_master_gateway(hass)
-    if CONF_BRIDGE_ID in data:
-        gateway = hass.data[DOMAIN][normalize_bridge_id(data[CONF_BRIDGE_ID])]
-
     device_registry, entity_registry = await asyncio.gather(
-        hass.helpers.device_registry.async_get_registry(),
-        hass.helpers.entity_registry.async_get_registry(),
+        gateway.hass.helpers.device_registry.async_get_registry(),
+        gateway.hass.helpers.entity_registry.async_get_registry(),
     )
 
     entity_entries = async_entries_for_config_entry(
