@@ -1,5 +1,4 @@
 """The Netatmo integration."""
-import asyncio
 import logging
 import secrets
 
@@ -20,7 +19,11 @@ from homeassistant.const import (
     EVENT_HOMEASSISTANT_STOP,
 )
 from homeassistant.core import CoreState, HomeAssistant
-from homeassistant.helpers import config_entry_oauth2_flow, config_validation as cv
+from homeassistant.helpers import (
+    aiohttp_client,
+    config_entry_oauth2_flow,
+    config_validation as cv,
+)
 from homeassistant.helpers.dispatcher import (
     async_dispatcher_connect,
     async_dispatcher_send,
@@ -41,6 +44,10 @@ from .const import (
     DOMAIN,
     OAUTH2_AUTHORIZE,
     OAUTH2_TOKEN,
+    PLATFORMS,
+    WEBHOOK_ACTIVATION,
+    WEBHOOK_DEACTIVATION,
+    WEBHOOK_PUSH_TYPE,
 )
 from .data_handler import NetatmoDataHandler
 from .webhook import async_handle_webhook
@@ -59,8 +66,6 @@ CONFIG_SCHEMA = vol.Schema(
     extra=vol.ALLOW_EXTRA,
 )
 
-PLATFORMS = ["camera", "climate", "light", "sensor"]
-
 
 async def async_setup(hass: HomeAssistant, config: dict):
     """Set up the Netatmo component."""
@@ -91,7 +96,7 @@ async def async_setup(hass: HomeAssistant, config: dict):
     return True
 
 
-async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
+async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Set up Netatmo from a config entry."""
     implementation = (
         await config_entry_oauth2_flow.async_get_config_entry_implementation(
@@ -103,18 +108,18 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
     if not entry.unique_id:
         hass.config_entries.async_update_entry(entry, unique_id=DOMAIN)
 
+    session = config_entry_oauth2_flow.OAuth2Session(hass, entry, implementation)
     hass.data[DOMAIN][entry.entry_id] = {
-        AUTH: api.ConfigEntryNetatmoAuth(hass, entry, implementation)
+        AUTH: api.AsyncConfigEntryNetatmoAuth(
+            aiohttp_client.async_get_clientsession(hass), session
+        )
     }
 
     data_handler = NetatmoDataHandler(hass, entry)
     await data_handler.async_setup()
     hass.data[DOMAIN][entry.entry_id][DATA_HANDLER] = data_handler
 
-    for platform in PLATFORMS:
-        hass.async_create_task(
-            hass.config_entries.async_forward_entry_setup(entry, platform)
-        )
+    hass.config_entries.async_setup_platforms(entry, PLATFORMS)
 
     async def unregister_webhook(_):
         if CONF_WEBHOOK_ID not in entry.data:
@@ -123,9 +128,15 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
         async_dispatcher_send(
             hass,
             f"signal-{DOMAIN}-webhook-None",
-            {"type": "None", "data": {"push_type": "webhook_deactivation"}},
+            {"type": "None", "data": {WEBHOOK_PUSH_TYPE: WEBHOOK_DEACTIVATION}},
         )
         webhook_unregister(hass, entry.data[CONF_WEBHOOK_ID])
+        try:
+            await hass.data[DOMAIN][entry.entry_id][AUTH].async_dropwebhook()
+        except pyatmo.ApiError:
+            _LOGGER.debug(
+                "No webhook to be dropped for %s", entry.data[CONF_WEBHOOK_ID]
+            )
 
     async def register_webhook(event):
         if CONF_WEBHOOK_ID not in entry.data:
@@ -146,9 +157,9 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
                 entry.data[CONF_WEBHOOK_ID]
             )
 
-        if entry.data["auth_implementation"] == "cloud" and not webhook_url.startswith(
-            "https://"
-        ):
+        if entry.data[
+            "auth_implementation"
+        ] == cloud.DOMAIN and not webhook_url.startswith("https://"):
             _LOGGER.warning(
                 "Webhook not registered - "
                 "https and port 443 is required to register the webhook"
@@ -166,7 +177,7 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
 
             async def handle_event(event):
                 """Handle webhook events."""
-                if event["data"]["push_type"] == "webhook_activation":
+                if event["data"][WEBHOOK_PUSH_TYPE] == WEBHOOK_ACTIVATION:
                     if activation_listener is not None:
                         activation_listener()
 
@@ -179,16 +190,16 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
                 handle_event,
             )
 
-            activation_timeout = async_call_later(hass, 10, unregister_webhook)
+            activation_timeout = async_call_later(hass, 30, unregister_webhook)
 
-            await hass.async_add_executor_job(
-                hass.data[DOMAIN][entry.entry_id][AUTH].addwebhook, webhook_url
-            )
+            await hass.data[DOMAIN][entry.entry_id][AUTH].async_addwebhook(webhook_url)
             _LOGGER.info("Register Netatmo webhook: %s", webhook_url)
         except pyatmo.ApiError as err:
             _LOGGER.error("Error during webhook registration - %s", err)
 
-        hass.bus.async_listen_once(EVENT_HOMEASSISTANT_STOP, unregister_webhook)
+        entry.async_on_unload(
+            hass.bus.async_listen_once(EVENT_HOMEASSISTANT_STOP, unregister_webhook)
+        )
 
     if hass.state == CoreState.running:
         await register_webhook(None)
@@ -198,27 +209,26 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
     hass.services.async_register(DOMAIN, "register_webhook", register_webhook)
     hass.services.async_register(DOMAIN, "unregister_webhook", unregister_webhook)
 
+    entry.add_update_listener(async_config_entry_updated)
+
     return True
 
 
+async def async_config_entry_updated(hass: HomeAssistant, entry: ConfigEntry) -> None:
+    """Handle signals of config entry being updated."""
+    async_dispatcher_send(hass, f"signal-{DOMAIN}-public-update-{entry.entry_id}")
+
+
 async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry):
     """Unload a config entry."""
     if CONF_WEBHOOK_ID in entry.data:
-        await hass.async_add_executor_job(
-            hass.data[DOMAIN][entry.entry_id][AUTH].dropwebhook
-        )
+        webhook_unregister(hass, entry.data[CONF_WEBHOOK_ID])
+        await hass.data[DOMAIN][entry.entry_id][AUTH].async_dropwebhook()
         _LOGGER.info("Unregister Netatmo webhook")
 
     await hass.data[DOMAIN][entry.entry_id][DATA_HANDLER].async_cleanup()
 
-    unload_ok = all(
-        await asyncio.gather(
-            *[
-                hass.config_entries.async_forward_entry_unload(entry, platform)
-                for platform in PLATFORMS
-            ]
-        )
-    )
+    unload_ok = await hass.config_entries.async_unload_platforms(entry, PLATFORMS)
 
     if unload_ok:
         hass.data[DOMAIN].pop(entry.entry_id)
